#include <assert.h>
#include <stdbool.h>
#include <stdio.h>
#include <stdint.h>
#include <stdlib.h>
#include <string.h>

#include <yubihsm.h>

#define DEFAULT_CONNECTOR_URL "http://127.0.0.1:12345"

const char *key_label = "label";
const uint8_t password1[] = "password";
const uint8_t password2[] = "letmein!";

int main(void) {
  yh_connector *connector = NULL;
  uint8_t context[YH_CONTEXT_LEN];
  yh_session *session = NULL;
  yh_rc yrc = YHR_GENERIC_ERROR;

  uint16_t keyset = 1;

  yrc = yh_init();
  assert(yrc == YHR_SUCCESS);

  yrc = yh_init_connector(DEFAULT_CONNECTOR_URL, &connector);
  assert(yrc == YHR_SUCCESS);

  yrc = yh_connect_best(&connector, 1, NULL);
  assert(yrc == YHR_SUCCESS);

  yrc =
    yh_create_session_derived(connector, keyset, password1, sizeof(password1),
                              false, context, sizeof(context), &session);
  assert(yrc == YHR_SUCCESS);

  yrc = yh_authenticate_session(session, context, sizeof(context));
  assert(yrc == YHR_SUCCESS);

  uint8_t session_id;
  yrc = yh_get_session_id(session, &session_id);
  assert(yrc == YHR_SUCCESS);

  printf("Successfully established session %02d using keyset %04x\n",
         session_id, keyset);

  yh_capabilities capabilities = {{0}};
  yrc = yh_capabilities_to_num("audit", &capabilities);
  assert(yrc == YHR_SUCCESS);

  uint16_t domain_five = 0;
  yrc = yh_parse_domains("5", &domain_five);
  assert(yrc == YHR_SUCCESS);

  uint16_t key_id = 0; // ID 0 lets the device generate an ID
  yrc = yh_util_import_authkey(session, &key_id, key_label, domain_five,
                               &capabilities, &capabilities, password2,
                               sizeof(password2));
  assert(yrc == YHR_SUCCESS);

  printf("Imported authkey with ID %04x\n", key_id);

  yrc = yh_util_close_session(session);
  assert(yrc == YHR_SUCCESS);

  yrc = yh_destroy_session(&session);
  assert(yrc == YHR_SUCCESS);

  printf("Closed session %02d\n", session_id);

  yrc =
    yh_create_session_derived(connector, key_id, password2, sizeof(password2),
                              false, context, sizeof(context), &session);
  assert(yrc == YHR_SUCCESS);

  yrc = yh_authenticate_session(session, context, sizeof(context));
  assert(yrc == YHR_SUCCESS);

  yrc = yh_get_session_id(session, &session_id);
  assert(yrc == YHR_SUCCESS);

  printf("Successfully established session %02d using keyset %04x\n",
         session_id, key_id);

  printf("Trying to get 16 bytes of random data\n");

  uint8_t data[16];
  size_t data_len = sizeof(data);
  yrc = yh_util_get_random(session, 16, data, &data_len);
  assert(yrc == YHR_DEVICE_INV_PERMISSION);

  printf("Unable to get random data: %s\n", yh_strerror(yrc));

  yrc = yh_util_close_session(session);
  assert(yrc == YHR_SUCCESS);

  yrc = yh_destroy_session(&session);
  assert(yrc == YHR_SUCCESS);

  yh_disconnect(connector);
  assert(yrc == YHR_SUCCESS);

  yrc = yh_exit();
  assert(yrc == YHR_SUCCESS);

  return 0;
}
